@@ -27,11 +27,11 @@ class ThreeValuedTruth:
         return self.val == ThreeValuedTruth.UNKNOWN
 
     @staticmethod
-    def from_bool(b: bool) -> 'ThreeValuedTruth':
+    def from_bool(b: bool) -> "ThreeValuedTruth":
         return ThreeValuedTruth(int(b))
 
     @staticmethod
-    def all(args: Iterable['ThreeValuedTruth']) -> 'ThreeValuedTruth':
+    def all(args: Iterable["ThreeValuedTruth"]) -> "ThreeValuedTruth":
         args = list(args)
         if any(elem.is_false() for elem in args):
             return ThreeValuedTruth.false()
@@ -40,7 +40,7 @@ class ThreeValuedTruth:
         return ThreeValuedTruth.true()
 
     @staticmethod
-    def any(args: Iterable['ThreeValuedTruth']) -> 'ThreeValuedTruth':
+    def any(args: Iterable["ThreeValuedTruth"]) -> "ThreeValuedTruth":
         args = list(args)
         if any(elem.is_true() for elem in args):
             return ThreeValuedTruth.true()
@@ -49,7 +49,7 @@ class ThreeValuedTruth:
         return ThreeValuedTruth.false()
 
     @staticmethod
-    def not_(arg: 'ThreeValuedTruth') -> 'ThreeValuedTruth':
+    def not_(arg: "ThreeValuedTruth") -> "ThreeValuedTruth":
         if arg.is_true():
             return ThreeValuedTruth.false()
         if arg.is_false():
@@ -74,19 +74,17 @@ class ThreeValuedTruth:
 
         return ThreeValuedTruth(bool(self))
 
-    def __and__(self, other: 'ThreeValuedTruth') -> 'ThreeValuedTruth':
+    def __and__(self, other: "ThreeValuedTruth") -> "ThreeValuedTruth":
         if self.is_unknown() or other.is_unknown():
             return ThreeValuedTruth.unknown()
 
         return ThreeValuedTruth.from_bool(bool(self) and bool(other))
 
-    def __or__(self, other: 'ThreeValuedTruth') -> 'ThreeValuedTruth':
+    def __or__(self, other: "ThreeValuedTruth") -> "ThreeValuedTruth":
         if self.is_unknown() or other.is_unknown():
             return ThreeValuedTruth.unknown()
 
         return ThreeValuedTruth.from_bool(bool(self) or bool(other))
 
     def __str__(self):
-        return ("TRUE" if self.is_true() else
-                "FALSE" if self.is_false() else
-                "UNKNOWN")
+        return "TRUE" if self.is_true() else "FALSE" if self.is_false() else "UNKNOWN"
